@@ -21,6 +21,7 @@
  */
 
 #include "libavutil/common.h"
+
 #include "parser.h"
 #include "hevc.h"
 #include "golomb.h"
@@ -36,7 +37,8 @@ typedef struct HEVCParseContext {
  * Find the end of the current frame in the bitstream.
  * @return the position of the first byte of the next frame, or END_NOT_FOUND
  */
-static int hevc_find_frame_end(AVCodecParserContext *s, const uint8_t *buf, int buf_size)
+static int hevc_find_frame_end(AVCodecParserContext *s, const uint8_t *buf,
+                               int buf_size)
 {
     int i;
     ParseContext *pc = &((HEVCParseContext *)s->priv_data)->pc;
@@ -82,9 +84,8 @@ static int hevc_find_frame_end(AVCodecParserContext *s, const uint8_t *buf, int
  * @param buf buffer with field/frame data.
  * @param buf_size size of the buffer.
  */
-static inline int parse_nal_units(AVCodecParserContext *s,
-                                  AVCodecContext *avctx,
-                                  const uint8_t *buf, int buf_size)
+static inline int parse_nal_units(AVCodecParserContext *s, AVCodecContext *avctx,
+                      const uint8_t *buf, int buf_size)
 {
     HEVCContext   *h  = &((HEVCParseContext *)s->priv_data)->h;
     GetBitContext *gb = &h->HEVClc->gb;
@@ -165,6 +166,8 @@ static inline int parse_nal_units(AVCodecParserContext *s,
         case NAL_IDR_N_LP:
         case NAL_CRA_NUT:
             sh->first_slice_in_pic_flag = get_bits1(gb);
+            s->picture_structure = h->picture_struct;
+            s->field_order = h->picture_struct;
 
             if (h->nal_unit_type >= 16 && h->nal_unit_type <= 23) {
                 s->key_frame = 1;
@@ -184,7 +187,7 @@ static inline int parse_nal_units(AVCodecParserContext *s,
             }
             if (h->sps != (HEVCSPS*)h->sps_list[h->pps->sps_id]->data) {
                 h->sps = (HEVCSPS*)h->sps_list[h->pps->sps_id]->data;
-                h->vps = h->vps_list[h->sps->vps_id];
+                h->vps = (HEVCVPS*)h->vps_list[h->sps->vps_id]->data;
             }
 
             if (!sh->first_slice_in_pic_flag) {
@@ -267,7 +270,7 @@ static int hevc_parse(AVCodecParserContext *s,
     } else {
         next = hevc_find_frame_end(s, buf, buf_size);
         if (ff_combine_frame(pc, next, &buf, &buf_size) < 0) {
-            *poutbuf = NULL;
+            *poutbuf      = NULL;
             *poutbuf_size = 0;
             return buf_size;
         }
@@ -275,7 +278,7 @@ static int hevc_parse(AVCodecParserContext *s,
 
     parse_nal_units(s, avctx, buf, buf_size);
 
-    *poutbuf = buf;
+    *poutbuf      = buf;
     *poutbuf_size = buf_size;
     return next;
 }
@@ -291,13 +294,12 @@ static int hevc_split(AVCodecContext *avctx, const uint8_t *buf, int buf_size)
         state = (state << 8) | buf[i];
         if (((state >> 8) & 0xFFFFFF) == START_CODE) {
             int nut = (state >> 1) & 0x3F;
-            if (nut >= NAL_VPS && nut <= NAL_PPS) {
+            if (nut >= NAL_VPS && nut <= NAL_PPS)
                 has_ps = 1;
-            } else if (has_ps) {
+            else if (has_ps)
                 return i - 3;
-            } else { // no parameter set at the beginning of the stream
+            else // no parameter set at the beginning of the stream
                 return 0;
-            }
         }
     }
     return 0;
@@ -323,12 +325,15 @@ static void hevc_close(AVCodecParserContext *s)
     av_freep(&pc->buffer);
 
     for (i = 0; i < FF_ARRAY_ELEMS(h->vps_list); i++)
-        av_freep(&h->vps_list[i]);
+        av_buffer_unref(&h->vps_list[i]);
     for (i = 0; i < FF_ARRAY_ELEMS(h->sps_list); i++)
         av_buffer_unref(&h->sps_list[i]);
     for (i = 0; i < FF_ARRAY_ELEMS(h->pps_list); i++)
         av_buffer_unref(&h->pps_list[i]);
 
+    av_buffer_unref(&h->current_sps);
+    h->sps = NULL;
+
     for (i = 0; i < h->nals_allocated; i++)
         av_freep(&h->nals[i].rbsp_buffer);
     av_freep(&h->nals);
