@@ -89,12 +89,13 @@ static void ff_acelp_interpolatef_mips(float *out, const float *in,
                 "addu   %[p_filter_coeffs_m],  %[p_filter_coeffs_m],   %[prec] \n\t"
                 "madd.s %[v],%[v],%[in_val_m], %[fc_val_m]                     \n\t"
 
-                : [v] "=&f" (v),[p_in_p] "+r" (p_in_p), [p_in_m] "+r" (p_in_m),
+                : [v] "+&f" (v),[p_in_p] "+r" (p_in_p), [p_in_m] "+r" (p_in_m),
                   [p_filter_coeffs_p] "+r" (p_filter_coeffs_p),
                   [in_val_p] "=&f" (in_val_p), [in_val_m] "=&f" (in_val_m),
                   [fc_val_p] "=&f" (fc_val_p), [fc_val_m] "=&f" (fc_val_m),
                   [p_filter_coeffs_m] "+r" (p_filter_coeffs_m)
                 : [prec] "r" (prec)
+                : "memory"
             );
         }
         out[n] = v;
@@ -201,7 +202,7 @@ static void ff_acelp_apply_order_2_transfer_function_mips(float *out, const floa
            [pole_coeffs] "r" (pole_coeffs)
          : "$f0", "$f1", "$f2", "$f3", "$f4", "$f5",
            "$f6", "$f7",  "$f8", "$f9", "$f10", "$f11",
-           "$f12", "$f13", "$f14", "$f15", "$f16"
+           "$f12", "$f13", "$f14", "$f15", "$f16", "memory"
     );
 }
 #endif /* HAVE_INLINE_ASM */
